# CheckNHDPlusV21Currentness.py
# Checks version files from local copy of NHDPlus V2.1 data against current
# version on Horizon Systems FTP site. Lists any version differences.

# History:
# Alan Rea, ahrea@usgs.gov, 9/23/2013, original coding
#                          10/18/2013, revised to ignore 3 data packaging errors,
#                                      and to ignore version files other than the most
#                                      recent for a component
# Curtis Price, cprice@usgs.gov 11/4/2014, removed import arcinfo (not needed)
#                                          added arcgis version check
#                                          added ftp fail message to user
# Cindy McKay, ldm@horizon-systems.com 12/4/2014, added GlobalData to folder list
#                                                 pointed ftp connection to EPA Amazon Cloud
# Harry Chriss, hec@horizon-systems.com 9/1/2015, updated folder list

import sys
import os
import traceback
import ftplib
import string

import arcinfo # Check out ArcGIS Advanced license
import arcpy
from arcpy import ExecuteError as GPError
#
import egis
# from/import commonly used egis methods
from egis import GPMsg, MsgError

try:

    # check for arc version
    try:
        type(arcpy.da)
    except:
        raise MsgError("This tool requires ArcGIS 10.1 SP1 or later")

    # get arguments
    localdir = arcpy.GetParameterAsText(0)

    GPMsg("\nChecking Horizon-Systems FTP server to get list of current data files...\n")

    dirlist = ['/NHDplus/NHDPlusV21/Data/NationalData/',
               '/NHDplus/NHDPlusV21/Data/GlobalData/',
			   '/NHDplus/NHDPlusV21/Data/Extensions/Accumulation/',
			   '/NHDplus/NHDPlusV21/Data/Extensions/NLCD2011LandUse/',
			   '/NHDplus/NHDPlusV21/Data/Extensions/SimplifiedCatchments/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusCA/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusCI/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusCO/NHDPlus14/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusCO/NHDPlus15/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusGB/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusGL/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusHI/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusMA/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusMS/NHDPlus05/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusMS/NHDPlus06/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusMS/NHDPlus07/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusMS/NHDPlus08/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusMS/NHDPlus10L/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusMS/NHDPlus10U/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusMS/NHDPlus11/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusNE/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusPI/NHDPlus22AS/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusPI/NHDPlus22GU/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusPI/NHDPlus22MP/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusPN/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusRG/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusSA/NHDPlus03N/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusSA/NHDPlus03S/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusSA/NHDPlus03W/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusSR/',
               '/NHDplus/NHDPlusV21/Data/NHDPlusTX/']

    # These files were originally distributed with the wrong version files.
    # Ignore them.
    ignorelist = ['NHDPlusV21_CO_15_WBDSnapshot_02',
                  'NHDPlusV21_MS_05_05a_CatSeed_01',
                  'NHDPlusV21_MS_11_EROMExtension_01']

    # Open FTP session
    try:
        ftp=ftplib.FTP("ec2-54-227-241-43.compute-1.amazonaws.com")
    except:
        raise MsgError("Failed to open FTP server.\n"
                       "If you have checked your internet connection and\n"
                       "can ping ec2-54-227-241-43.compute-1.amazonaws.com, please contact\n"
                       "NHDplus@hscnet.com\n")

    ftp.login("anonymous","anonymous@")

    filelist = []
    for eachdir in dirlist:
        ftp.cwd(eachdir)
        filelist += ftp.nlst()

    GPMsg("Getting list of local files that have been extracted under " + localdir)
    GPMsg("\nComparing lists...\n")

    # Walk local directory tree, collecting filenames, if they are in the list, compare
    n = 0
    dictNames = {}
    dictList = []
    for dirpath, dirnames, filenames in arcpy.da.Walk(localdir):
        for fname in filenames:
            fnmext = fname.split('.')
            name = fnmext[0]
            if name[0:10] == 'NHDPlusV21':
                bname = name[:-2]
                vernum = name[-2:]
                if dictNames.has_key(bname):       # entry already exists in dictionary
                    if dictNames[bname] < vernum:  # version number is less than current record
                        dictNames[bname] = vernum  # update version number with current
                else:                              # entry does not yet exist in dictionary
                    dictNames[bname] = vernum      # make entry in dictionary using current bname and vernum

    for k, v in dictNames.iteritems():             # convert dictionary into sorted list
        temp = str(k) + str(v)
        dictList.append(temp)
        dictList.sort()

    for fname in dictList:                         # process each version file found in local directory tree
        try:
            fnmext = fname.split('.')
            name = fnmext[0]
            if name[0:10] == 'NHDPlusV21':
                for srvnm in filelist:
                    if srvnm[0:10] == 'NHDPlusV21':
                        lclname = name[:-2]
                        srvname = srvnm[:-5]
                        srvnmver = srvnm[:-3]
                        if lclname == srvname and srvnmver not in ignorelist:
                            if name <> srvnmver:
                                GPMsg("w", (
                                    " local version: {} does not match\n"
                                    " server version: {}\n").format(
                                        name, srvnmver))
                                n += 1

        except:
            pass
    if n > 0:
        GPMsg("w", """

Found {} components that differ from Horizon Systems server version
Please read the User Guide sections on Distribution Files and the
NHDPlus V21 versioning system.

Also be sure to download an updated copy of the Release Notes
for each updated VPU.
""".format(n))
    else:
        GPMsg("""
Your local NHDPlus V2.1 data are all up to date!
""")

# handle errors and report

except MsgError, xmsg:
    GPMsg("e", str(xmsg))
except Exception, msg:
    # script failed - report line number
    tbinfo = traceback.format_tb(sys.exc_info()[2])[0]
    GPMsg("e", tbinfo.strip())
    GPMsg("e", str(msg))
