# PrepNHDPlusV2Data.py - Builds spatial and attribute indexes and raster pyramids
#                        for any NHDPlusV2 datasets it discovers under the user-
#                        specified folder.

# History:
# Alan Rea, ahrea@usgs.gov, 2013-08-28, original coding

import sys
import os
import traceback
import string

import arcpy
import arcinfo # Check out ArcGIS Advanced license
## arcpy.CheckOutExtension("spatial") # spatial analyst license
## from arcpy.sa import * # for ArcPy map algebra
from arcpy import ExecuteError as GPError
#
import egis
# from/import commonly used egis methods
from egis import GPMsg, MsgError

try:
    
    # get arguments
    startdir = arcpy.GetParameterAsText(0)
    BuildPyr = arcpy.GetParameterAsText(1) == "true"
    
    arcpy.env.workspace = startdir
    
    # List of shapes and fields to index if (any).
    # If second element is left null, no fields will be indexed
    ShapeProcList = [
      ["BoundaryUnit.shp",""],
      ["GageLoc.shp","Source_Fea"],
      ["SuperCatchments.shp","FeatureID;GRIDCODE"],
      ["BoundaryUnit.shp",""],
      ["NationalWBDSnapshot.shp","HUC_8;HUC_12"],
      ["Catchment.shp","FeatureID;GRIDCODE"],
      ["BurnAddWaterbody.shp","PolyID"],
      ["BurnAddLine.shp","FeatureID;GRIDCODE"],
      ["BurnWaterbody.shp","ComID"],
      ["LandSea.shp","LandSeaID"],
      ["Sink.shp","SinkID;FeatureID;GRIDCODE"],
      ["Wall.shp","WallID"],
      ["NHDFlowline.shp","ComID"],
      ["NHDWaterbody.shp","ComID"],
      ["NHDPoint.shp","ComID"],
      ["NHDLine.shp","ComID"],
      ["NHDArea.shp","ComID"]]

    # List of tables with fields to index (if any)
    TableProcList = [
      ["BoundaryValue.dbf","FeatureID;GRIDCODE"],
      ["GageInfo.dbf","GageID"],
      ["Gage_Smooth.dbf","SITE_NO"],
      ["CumulativeArea.dbf","ComID"],
      ["DivFracMP.dbf","ComID"],
      ["ElevSlope.dbf","ComID"],
      ["HeadwaterNodeArea.dbf","ComID"],
      ["PlusARPointEvent.dbf","ComID"],
      ["PlusFlowlineVAA.dbf","ComID"],
      ["PlusFlow.dbf","FromComID;ToComID;FromHydSeq;ToHydSeq;FromLvlPat;ToLvlPat"],
      ["PlusFlowAR.dbf","FromComID;ToComID"],
      ["MegaDiv.dbf","FromComID;ToComID"],
      ["FeatureIDGridCode.dbf","FeatureID;GRIDCODE"],
      ["BurnLineEvent.dbf","ComID;GRIDCODE"],
      ["EROM_MA0001.dbf","ComID"],
      ["VogelFlow.dbf","ComID"],
      ["EROMQA_MA0001.dbf","ComID"]]

    # List of rasters to build pyramids for
    GridProcList = [
        ["nationalcat", "NEAREST"], # maybe too big to process?? 
        ["cat", "NEAREST"],
        ["fdr", "NEAREST"],
        ["fac", "BILINEAR"],
        ["elev_cm", "BILINEAR"],
        ["shdrelief", "BILINEAR"],
        ["fdrnull", "NEAREST"],
        ["filledareas", "NEAREST"],
        ["catseed", "NEAREST"],
        ["hydrodem", "BILINEAR"]]


    for dirpath, dirnames, filenames in arcpy.da.Walk(startdir):
        for fname in filenames:
            # Create spatial indexes and attribute indexes for shape files
            for shp in ShapeProcList:
                if fname.lower() == shp[0].lower():
                    GPMsg("Building Spatial Index for " + os.path.join(dirpath, fname))
                    arcpy.AddSpatialIndex_management(os.path.join(dirpath, fname))
                    if shp[1]!="":
                        GPMsg("Building Attribute Index " + os.path.join(dirpath, fname) + ": " + shp[1])
                        arcpy.AddIndex_management(os.path.join(dirpath, fname), shp[1])
                        
            # Create spatial indexes and attribute indexes for dbf tables
            for tbl in TableProcList:
                if fname.lower() == tbl[0].lower():
                    if tbl[1]!="":
                        GPMsg("Building Attribute Index " + os.path.join(dirpath, fname) + ": " + tbl[1])
                        arcpy.AddIndex_management(os.path.join(dirpath, fname), tbl[1])
            
            # Create pyramids for rasters if BuildPyr == "true"
            for grd in GridProcList:
                if BuildPyr:
                    if fname.lower() == grd[0].lower():
                        if grd[1]!="":
                            GPMsg("Building Pyramid for " + os.path.join(dirpath, fname) + ": " + grd[1] + " resampling, skipping if already exists")
                            arcpy.BuildPyramids_management(os.path.join(dirpath, fname), "", "", grd[1], "", "", "SKIP_EXISTING")

# handle errors and report

except MsgError, xmsg:
    GPMsg("e",str(xmsg))
except GPError:
    tbinfo = traceback.format_tb(sys.exc_info()[2])[0]
    GPMsg("e",tbinfo.strip())
    numMsg = arcpy.GetMessageCount()
    # Using "return" passes along severity and links
    for i in range(0, numMsg):
        GPMsg("Return",i)
except Exception, msg:
    # generic python error
    tbinfo = traceback.format_tb(sys.exc_info()[2])[0]
    GPMsg("e",tbinfo.strip())
    GPMsg("e",str(msg))
finally:
    # Clean up here (delete cursors, temp files)
    pass

